'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Asset = require('../Asset');
var _parse = require('posthtml-parser');
var api = require('posthtml/lib/api');
var urlJoin = require('../utils/urlJoin');
var render = require('posthtml-render');
var posthtmlTransform = require('../transforms/posthtml');
var isURL = require('../utils/is-url');

// A list of all attributes that may produce a dependency
// Based on https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes
var ATTRS = {
  src: ['script', 'img', 'audio', 'video', 'source', 'track', 'iframe', 'embed'],
  href: ['link', 'a'],
  poster: ['video'],
  'xlink:href': ['use'],
  content: ['meta']
};

// A list of metadata that should produce a dependency
// Based on:
// - http://schema.org/
// - http://ogp.me
// - https://developer.twitter.com/en/docs/tweets/optimize-with-cards/overview/markup
// - https://msdn.microsoft.com/en-us/library/dn255024.aspx
var META = {
  property: ['og:image', 'og:image:url', 'og:image:secure_url', 'og:audio', 'og:audio:secure_url', 'og:video', 'og:video:secure_url'],
  name: ['twitter:image', 'msapplication-square150x150logo', 'msapplication-square310x310logo', 'msapplication-square70x70logo', 'msapplication-wide310x150logo', 'msapplication-TileImage'],
  itemprop: ['image', 'logo', 'screenshot', 'thumbnailUrl', 'contentUrl', 'downloadUrl']
};

var HTMLAsset = function (_Asset) {
  (0, _inherits3.default)(HTMLAsset, _Asset);

  function HTMLAsset(name, pkg, options) {
    (0, _classCallCheck3.default)(this, HTMLAsset);

    var _this = (0, _possibleConstructorReturn3.default)(this, (HTMLAsset.__proto__ || (0, _getPrototypeOf2.default)(HTMLAsset)).call(this, name, pkg, options));

    _this.type = 'html';
    _this.isAstDirty = false;
    return _this;
  }

  (0, _createClass3.default)(HTMLAsset, [{
    key: 'parse',
    value: function parse(code) {
      var res = _parse(code, { lowerCaseAttributeNames: true });
      res.walk = api.walk;
      res.match = api.match;
      return res;
    }
  }, {
    key: 'processSingleDependency',
    value: function processSingleDependency(path) {
      var assetPath = this.addURLDependency(decodeURIComponent(path));
      if (!isURL(assetPath)) {
        assetPath = urlJoin(this.options.publicURL, assetPath);
      }
      return assetPath;
    }
  }, {
    key: 'collectSrcSetDependencies',
    value: function collectSrcSetDependencies(srcset) {
      var newSources = [];
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(srcset.split(',')), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var source = _step.value;

          var pair = source.trim().split(' ');
          if (pair.length === 0) continue;
          pair[0] = this.processSingleDependency(pair[0]);
          newSources.push(pair.join(' '));
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return newSources.join(',');
    }
  }, {
    key: 'collectDependencies',
    value: function collectDependencies() {
      var _this2 = this;

      this.ast.walk(function (node) {
        if (node.attrs) {
          if (node.tag === 'meta') {
            if (!(0, _keys2.default)(node.attrs).some(function (attr) {
              var values = META[attr];
              return values && values.includes(node.attrs[attr]);
            })) {
              return node;
            }
          }

          for (var attr in node.attrs) {
            if (node.tag === 'img' && attr === 'srcset') {
              node.attrs[attr] = _this2.collectSrcSetDependencies(node.attrs[attr]);
              _this2.isAstDirty = true;
              continue;
            }
            var elements = ATTRS[attr];
            // Check for virtual paths
            if (node.tag === 'a' && node.attrs[attr].lastIndexOf('.') < 1) {
              continue;
            }
            if (elements && elements.includes(node.tag)) {
              node.attrs[attr] = _this2.processSingleDependency(node.attrs[attr]);
              _this2.isAstDirty = true;
            }
          }
        }

        return node;
      });
    }
  }, {
    key: 'pretransform',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return posthtmlTransform(this);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function pretransform() {
        return _ref.apply(this, arguments);
      }

      return pretransform;
    }()
  }, {
    key: 'generate',
    value: function generate() {
      var html = this.isAstDirty ? render(this.ast) : this.contents;
      return { html: html };
    }
  }]);
  return HTMLAsset;
}(Asset);

module.exports = HTMLAsset;